/*
 * $Id: AccessBtonic.java,v 1.1.1.1 2002/08/14 12:43:10 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package abagames.thiy;

import java.io.*;
import java.net.*;
import java.util.Random;

/**
 * Access BTONIC with SOAP Interface.
 *
 * @version $Revision: 1.1.1.1 $
 */
public class AccessBtonic {

  private final String DEFAULT_SERVICE_URL =
    "http://btonic.est.co.jp/NetDic/NetDicV06.asmx";

  private Random rnd = new Random();

  public String search(String query)
   throws MalformedURLException, IOException, UnsupportedEncodingException {
    URL url = new URL(DEFAULT_SERVICE_URL);
    URLConnection conn = url.openConnection();
    conn.setDoOutput(true);
    conn.setRequestProperty("Content-Type", "text/xml; charset=utf-8");

    /*conn.setRequestProperty("SOAPAction", "\"http://btonic.est.co.jp/NetDic/NetDicV06/GetDicList\"");
    PrintWriter pst = new PrintWriter(new OutputStreamWriter(conn.getOutputStream(), "UTF-8"));
    pst.print("<?xml version=\"1.0\" encoding=\"utf-8\"?><soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"> <soap:Body>");
    pst.print("<GetDicList xmlns=\"http://btonic.est.co.jp/NetDic/NetDicV06\" />");
    pst.print("</soap:Body> </soap:Envelope>");*/

    conn.setRequestProperty("SOAPAction", "\"http://btonic.est.co.jp/NetDic/NetDicV06/SearchDicItem\"");
    PrintWriter pst = new PrintWriter(new OutputStreamWriter(conn.getOutputStream(), "UTF-8"));
    pst.print("<?xml version=\"1.0\" encoding=\"utf-8\"?> <soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"> <soap:Body>");
    pst.print("<SearchDicItem xmlns=\"http://btonic.est.co.jp/NetDic/NetDicV06\">");
    pst.print("<DicID>c37c56a8-dbbb-4d16-992a-be348d40e613</DicID>");
    //pst.print("<DicID>3a157a62-f2a3-44b1-aac2-cf341569fc38</DicID>");
    pst.print("<QueryString>");
    pst.print(query);
    pst.print("</QueryString>");
    //pst.print("<ScopeOption>HEADWORD or KEYWORD or ANYWHERE</ScopeOption>");
    pst.print("<ScopeOption>ANYWHERE</ScopeOption>");
    //pst.print("<MatchOption>EXACT or STARTWITH or ENDWITH or CONTAIN</MatchOption>");
    pst.print("<MatchOption>CONTAIN</MatchOption>");
    //pst.print("<FormatOption>TEXT or HTML or DICX</FormatOption>");
    pst.print("<FormatOption>HTML</FormatOption>");
    pst.print("<ResourceOption>URI</ResourceOption>");
    pst.print("<CharsetOption>UNICODE</CharsetOption>");
    pst.print("<ReqItemIndex>");
    pst.print(Math.abs(rnd.nextInt())%10000);
    pst.print("</ReqItemIndex>");
    pst.print("<ReqItemTitleCount>1</ReqItemTitleCount>");
    pst.print("<ReqItemContentCount>1</ReqItemContentCount>");
    pst.print("</SearchDicItem> </soap:Body> </soap:Envelope>");

    pst.close();
    conn.connect();

    StringBuffer resultBuffer = new StringBuffer();
    BufferedReader rsp = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));
    for ( ; ; ) {
      String rspStr = rsp.readLine();
      if ( rspStr == null ) break;
      resultBuffer.append(rspStr);
    }
    rsp.close();

    return resultBuffer.toString();
  }

  private String removeDotNum(String str) {
    StringBuffer buf = new StringBuffer();
    for ( int i=0 ; i<str.length() ; i++ ) {
      char c = str.charAt(i);
      if ( c != 'E' &&
        c !='1' && c !='2' && c !='3' && c !='4' && c !='5' &&
        c !='6' && c !='7' && c !='8' && c !='9' ) buf.append(c);
    }
    return buf.toString();
  }

  private String removeTri(String str) {
    StringBuffer buf = new StringBuffer();
    for ( int i=0 ; i<str.length() ; i++ ) {
      char c = str.charAt(i);
      if ( c != '' && c != '' ) buf.append(c);
    }
    return buf.toString();
  }

  public String term, kana, mean;
  private String topStr;

  private String getSurStr(String str, String st, String ed) {
    int idx = str.indexOf(st);
    if ( idx != -1 ) {
      topStr = removeDotNum(str.substring(0, idx));
      return str.substring(idx+1, str.indexOf(ed, idx+1));
    } else {
      return "";
    }
  }

  private String getJapaneseTerm(String title) {
    String term;
    term = getSurStr(title, "F", "F");
    if ( term.equals("") ) {
      term = getSurStr(title, "m", "n");
    }
    kana = topStr;
    String brkTerm;
    brkTerm = getSurStr(term, "o", "p");
    if ( !brkTerm.equals("") ) term = brkTerm;
    term = removeTri(term);
    int slIdx = term.indexOf('/');
    if ( slIdx != -1 ) {
      term = term.substring(0, slIdx);
    }
    return term;
  }

  private String getMean(String str) {
    int prIdx = str.indexOf('D');
    if ( prIdx == -1 ) return str;
    String mean = str.substring(0, prIdx);
    int spIdx = mean.lastIndexOf(' ');
    if ( spIdx != -1 ) {
      mean = mean.substring(spIdx+1);
    }
    return XMLHandler.removeElement(mean, "small");
  }

  private final String[] querySeed = {
    "", "", "", "", "", "", "", "", "", ""
  };

  public void getRandomTerm() throws UnsupportedEncodingException, IOException {
    do {
      String result =
        search(querySeed[Math.abs(rnd.nextInt())%querySeed.length]);
      String title = XMLHandler.getElement(result, "Title");
      term = getJapaneseTerm(title);
      mean = getMean(XMLHandler.getElement(result, "div", "class=\"NetDicMeaning\""));
    } while ( term.equals("") );
  }

  public static void main(String[] args) {
    try {
      AccessBtonic accessBtonic = new AccessBtonic();
      AccessGoogle accessGoogle = new AccessGoogle();
      int termCnt = 0;
      for ( ; ; ) {
        accessBtonic.getRandomTerm();
        String result = accessGoogle.search(accessBtonic.term);
        int cnt = new Integer(XMLHandler.getElement(result, "estimatedTotalResultsCount")).intValue();
        if ( cnt < 1000 ) {
          System.out.println(accessBtonic.kana + " "  + accessBtonic.mean);
          //System.out.println("["+ accessBtonic.term + "] " + accessBtonic.mean);
          //System.out.println(" " + cnt + " hits");
          termCnt++;
        }
        if ( termCnt >= 10 ) return;
      }
    } catch ( Exception e ) {
      e.printStackTrace();
    }
  }
}
